@@ -26,7 +26,11 @@
 K_PLUGIN_CLASS_WITH_JSON(BalooFilePropertiesPlugin, "baloofilepropertiesplugin.json")
 
 BalooFilePropertiesPlugin::BalooFilePropertiesPlugin(QObject *parent, const QList<QVariant> &args)
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     : KPropertiesDialogPlugin(qobject_cast<KPropertiesDialog *>(parent))
+#else
+    : KPropertiesDialogPlugin(parent)
+#endif
 {
     Q_UNUSED(args);
 
