@@ -15,45 +15,42 @@
 using namespace Baloo;
 
 TagCheckBox::TagCheckBox(const QString &tag, QWidget *parent)
-    : QWidget(parent)
+    : QLabel(parent)
     , m_tag(tag)
 {
-    auto layout = new QHBoxLayout(this);
-    layout->setContentsMargins(0, 0, 0, 0);
+    auto lastSegment = tag.split(QLatin1Char('/'), Qt::SkipEmptyParts).last();
+    setText(lastSegment);
 
-    m_label = new QLabel(tag.split(QLatin1Char('/'), Qt::SkipEmptyParts).last(), this);
-    m_label->setToolTip(tag);
-    m_label->setMouseTracking(true);
-    m_label->setTextFormat(Qt::PlainText);
-    m_label->setForegroundRole(parent->foregroundRole());
-    m_child = m_label;
+    setToolTip(tag);
+    setTextFormat(Qt::PlainText);
+    setForegroundRole(parent->foregroundRole());
 
-    m_child->installEventFilter(this);
-    m_child->setMouseTracking(true);
-    layout->addWidget(m_child);
+    installEventFilter(this);
 }
 
 TagCheckBox::~TagCheckBox() = default;
 
-void TagCheckBox::leaveEvent(QEvent *event)
-{
-    QWidget::leaveEvent(event);
-    enableUrlHover(false);
-}
-
 bool TagCheckBox::eventFilter(QObject *watched, QEvent *event)
 {
-    if (watched == m_child) {
+    if (watched == this) {
         switch (event->type()) {
+        case QEvent::Enter: {
+            enableUrlHover(true);
+            break;
+        }
+        case QEvent::Leave: {
+            enableUrlHover(false);
+            break;
+        }
         case QEvent::MouseMove: {
             auto me = static_cast<QMouseEvent *>(event);
-            enableUrlHover(tagRect().contains(me->pos()));
+            enableUrlHover(rect().contains(me->pos()));
             break;
         }
 
         case QEvent::MouseButtonRelease: {
             auto me = static_cast<QMouseEvent *>(event);
-            if (me->button() == Qt::LeftButton && tagRect().contains(me->pos())) {
+            if (me->button() == Qt::LeftButton && rect().contains(me->pos())) {
                 Q_EMIT tagClicked(m_tag);
                 return true;
             }
@@ -69,20 +66,14 @@ bool TagCheckBox::eventFilter(QObject *watched, QEvent *event)
     return QWidget::eventFilter(watched, event);
 }
 
-QRect TagCheckBox::tagRect() const
-{
-    return QRect(QPoint(0, 0), m_label->size());
-}
-
 void TagCheckBox::enableUrlHover(bool enable)
 {
     if (m_urlHover != enable) {
         m_urlHover = enable;
         QFont f = font();
-        if (enable)
-            f.setUnderline(true);
-        m_child->setFont(f);
-        m_child->setCursor(enable ? Qt::PointingHandCursor : Qt::ArrowCursor);
+        f.setUnderline(enable);
+        setFont(f);
+        setCursor(enable ? Qt::PointingHandCursor : Qt::ArrowCursor);
     }
 }
 
