@@ -7,16 +7,11 @@
 #ifndef _BALOO_TAG_CHECKBOX_H_
 #define _BALOO_TAG_CHECKBOX_H_
 
-#include <QWidget>
-
-#include "tagwidget_p.h"
-
-class QMouseEvent;
-class QLabel;
+#include <QLabel>
 
 namespace Baloo
 {
-class TagCheckBox : public QWidget
+class TagCheckBox : public QLabel
 {
     Q_OBJECT
 
@@ -33,19 +28,13 @@ Q_SIGNALS:
     void tagClicked(const QString &tag);
 
 protected:
-    void leaveEvent(QEvent *event) override;
     bool eventFilter(QObject *watched, QEvent *event) override;
 
 private:
-    QRect tagRect() const;
     void enableUrlHover(bool enabled);
 
-    // two modes: checkbox and simple label
-    QLabel *m_label;
-    QWidget *m_child;
-
     QString m_tag;
-    bool m_urlHover;
+    bool m_urlHover = false;
 };
 }
 
