@@ -9,21 +9,43 @@
 #include "kblocklayout.h"
 #include "kedittagsdialog_p.h"
 #include "tagcheckbox.h"
-#include "tagwidget_p.h"
 
 #include <KLocalizedString>
 
 #include <QLabel>
-#include <QPushButton>
+#include <QMap>
 
 using namespace Baloo;
 
+class Baloo::TagWidgetPrivate
+{
+public:
+    void init(TagWidget *parent);
+    void rebuild();
+    void buildTagHash(const QStringList &tags);
+
+    /// create clickable tag label
+    void addTagLabel(const QString &tag);
+
+    /// check the corresponding checkboxes and even
+    /// add missing checkboxes
+    void selectTags(const QStringList &tags);
+
+    bool m_readOnly = false;
+
+    QMap<QString, TagCheckBox *> m_tagLabels;
+    QLabel *m_showAllLinkLabel = nullptr;
+    KBlockLayout *m_flowLayout = nullptr;
+    TagWidget *q;
+
+    void showEditDialog();
+    void kEditTagDialogFinished(int result);
+    KEditTagsDialog *m_editTagsDialog = nullptr;
+};
+
 void TagWidgetPrivate::init(TagWidget *parent)
 {
     q = parent;
-    m_readOnly = false;
-    m_showAllLinkLabel = nullptr;
-    m_editTagsDialog = nullptr;
 
     auto mainLayout = new QGridLayout(q);
     mainLayout->setContentsMargins(0, 0, 0, 0);
@@ -40,11 +62,11 @@ void TagWidgetPrivate::rebuild()
 
 void TagWidgetPrivate::buildTagHash(const QStringList &tags)
 {
-    qDeleteAll(m_checkBoxHash);
-    m_checkBoxHash.clear();
+    qDeleteAll(m_tagLabels);
+    m_tagLabels.clear();
 
     for (const QString &tag : tags) {
-        getTagCheckBox(tag);
+        addTagLabel(tag);
     }
 
     delete m_showAllLinkLabel;
@@ -57,29 +79,23 @@ void TagWidgetPrivate::buildTagHash(const QStringList &tags)
     m_showAllLinkLabel = new QLabel(q);
     m_flowLayout->addWidget(m_showAllLinkLabel);
     if (m_readOnly) {
+        m_showAllLinkLabel->setTextFormat(Qt::PlainText);
         m_showAllLinkLabel->setText(QStringLiteral("-"));
     } else {
-        QFont f(q->font());
-        f.setUnderline(true);
-        m_showAllLinkLabel->setFont(f);
-        m_showAllLinkLabel->setText(QLatin1String("<a href=\"add_tags\">") + (m_checkBoxHash.isEmpty() ? i18nc("@label", "Add...") : i18nc("@label", "Edit..."))
-                                    + QLatin1String("</a>"));
-        q->connect(m_showAllLinkLabel, SIGNAL(linkActivated(QString)), SLOT(slotShowAll()));
+        m_showAllLinkLabel->setTextFormat(Qt::RichText);
+        m_showAllLinkLabel->setText(QStringLiteral("<a href=\"add_tags\">%1</a>").arg(m_tagLabels.isEmpty() ? i18nc("@label", "Add...") : i18nc("@label", "Edit...")));
+        q->connect(m_showAllLinkLabel, &QLabel::linkActivated, [this]{showEditDialog();});
     }
 }
 
-TagCheckBox *TagWidgetPrivate::getTagCheckBox(const QString &tag)
+void TagWidgetPrivate::addTagLabel(const QString &tag)
 {
-    QMap<QString, TagCheckBox *>::iterator it = m_checkBoxHash.find(tag);
-    if (it == m_checkBoxHash.end()) {
-        // kDebug() << "Creating checkbox for" << tag.genericLabel();
-        auto checkBox = new TagCheckBox(tag, q);
-        q->connect(checkBox, SIGNAL(tagClicked(QString)), SIGNAL(tagClicked(QString)));
-        m_checkBoxHash.insert(tag, checkBox);
-        m_flowLayout->addWidget(checkBox);
-        return checkBox;
-    } else {
-        return it.value();
+    const auto it = m_tagLabels.find(tag);
+    if (it == m_tagLabels.end()) {
+        auto label = new TagCheckBox(tag, q);
+        q->connect(label, &TagCheckBox::tagClicked, q, &TagWidget::tagClicked);
+        m_tagLabels.insert(tag, label);
+        m_flowLayout->addWidget(label);
     }
 }
 
@@ -100,12 +116,7 @@ TagWidget::~TagWidget() = default;
 
 QStringList TagWidget::selectedTags() const
 {
-    QStringList tags;
-    QMapIterator<QString, TagCheckBox *> it(d->m_checkBoxHash);
-    while (it.hasNext()) {
-        tags << it.next().key();
-    }
-    return tags;
+    return d->m_tagLabels.keys();
 }
 
 Qt::Alignment TagWidget::alignment() const
@@ -134,28 +145,23 @@ void TagWidget::setReadyOnly(bool readOnly)
     d->rebuild();
 }
 
-void TagWidget::slotTagUpdateDone()
+void TagWidgetPrivate::showEditDialog()
 {
-    setEnabled(true);
+    m_editTagsDialog = new KEditTagsDialog(m_tagLabels.keys(), q);
+    m_editTagsDialog->setWindowModality(Qt::ApplicationModal);
+    q->connect(m_editTagsDialog, &QDialog::finished, q, [this](int result) {kEditTagDialogFinished(result);});
+    m_editTagsDialog->open();
 }
 
-void TagWidget::slotShowAll()
-{
-    d->m_editTagsDialog = new KEditTagsDialog(selectedTags(), this);
-    d->m_editTagsDialog->setWindowModality(Qt::ApplicationModal);
-    connect(d->m_editTagsDialog, SIGNAL(finished(int)), this, SLOT(slotKEditTagDialogFinished(int)));
-    d->m_editTagsDialog->open();
-}
-
-void TagWidget::slotKEditTagDialogFinished(int result)
+void TagWidgetPrivate::kEditTagDialogFinished(int result)
 {
     if (result == QDialog::Accepted) {
-        setSelectedTags(d->m_editTagsDialog->tags());
-        Q_EMIT selectionChanged(selectedTags());
+        selectTags(m_editTagsDialog->tags());
+        Q_EMIT q->selectionChanged(m_tagLabels.keys());
     }
 
-    d->m_editTagsDialog->deleteLater();
-    d->m_editTagsDialog = nullptr;
+    m_editTagsDialog->deleteLater();
+    m_editTagsDialog = nullptr;
 }
 
 #include "moc_tagwidget.cpp"
